/*
 *
 *  BlueZ - Bluetooth protocol stack for Linux
 *
 *  Copyright (C) 2007       Access Systems Inc. <http://www.access-company.com>
 *  Copyright (C) 2004-2007  Marcel Holtmann <marcel@holtmann.org>
 *
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <dlfcn.h>
#include <errno.h>
#include <string.h>

#include "hf_plugin.h"
#include "logging.h"

struct hf_service {
	int (*ag_state_changed_cb)(int state);
	int (*ag_indicate_call_cb)(int incoming);
	int (*ag_set_indicator_cb)(hf_indicator_t indicator, int value);
	int (*ag_cancel_call_cb)();
	int (*ag_event_cb)(int event, void *param);
};

struct hf_ag {
	struct hf_service service;
	void *handle;
	void *data;
	int (* init)(void **plg);
	void (* exit)(void *plg);
	void (* state_changed)(int state, int type);
};

void* hf_plugin_init(char* filename)
{
	struct hf_ag *plg;
	char funcname[512];
	char *basename, *dot;

	if(!filename || !*filename)
		return NULL;

	plg = malloc(sizeof(struct hf_ag));
	if(!plg)
		return NULL;

	memset(plg, 0, sizeof(struct hf_ag));

	info("%s(%s)", __FUNCTION__, filename);

	plg->handle = dlopen (filename, RTLD_NOW);
	if(!plg->handle)
		goto failed;

	basename = strrchr(filename, '/');
	if(basename)
		basename++;
	else
		basename = filename;

	if(strncmp(basename, "lib", 3)==0)
		basename += 3;

	dot = strrchr(basename, '.');
	if(dot)
		*dot = '\0';

	sprintf(funcname, "%s_exit", basename);
	debug("loading func %s", funcname);
	plg->exit = dlsym(plg->handle, funcname);
	if(!plg->exit)
		goto failed;

	sprintf(funcname, "%s_init", basename);
	debug("loading func %s", funcname);
	plg->init = dlsym(plg->handle, funcname);
	if(!plg->init)
		goto failed;

	sprintf(funcname, "%s_state_changed", basename);
	debug("loading func %s", funcname);
	plg->state_changed = dlsym(plg->handle, funcname);
	if(!plg->state_changed)
		goto failed;

	if(plg->init(&plg->data)<0)
		goto failed;

	debug("%s successful", funcname);

	return plg;

  failed:
	error("plugin load failed (error:%s, dlerror:%s)", strerror(errno), dlerror());
	hf_plugin_exit(plg);

	return NULL;
}

void hf_plugin_exit(void *plugin)
{
	struct hf_ag *plg = (struct hf_ag*)plugin;

	info("%s", __FUNCTION__);

	if(!plg)
		return;

	if(plg->exit)
		plg->exit(plg->data);

	if(plg->handle)
		dlclose (plg->handle); 

	free(plg);
}

void hf_plugin_state_changed(void *plugin, int state, int type)
{
	struct hf_ag *plg = (struct hf_ag*)plugin;

	info("%s(%d, %d)", __FUNCTION__, state, type);

	if(plg->state_changed)
		plg->state_changed(state, type);
}


