/*
 *
 *  BlueZ - Bluetooth protocol stack for Linux
 *
 *  Copyright (C) 2007       Access Systems Inc. <http://www.access-company.com>
 *  Copyright (C) 2004-2007  Marcel Holtmann <marcel@holtmann.org>
 *
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <fcntl.h>
#include <glib.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <unistd.h>
#include <errno.h>
#include <dbus/dbus-glib.h>
#include <dbus/dbus-glib-lowlevel.h>
#include <bluetooth/bluetooth.h>
#include <bluetooth/sdp.h>
#include <bluetooth/sdp_lib.h>

#include "device.h"
#include "manager.h"
#include "headset.h"
#include "hf_plugin.h"
#include "hf_plugin_alp.h"

/***********************************************************************
 *
 *	Methods implementation
 *
 ***********************************************************************/


/***********************************************************************
 *
 *	DBUS Method Handler
 *
 ***********************************************************************/

///////////////////////////////////////////////////////////////////////////////
static DBusHandlerResult dbus_handler_HSHF_func(DBusConnection * connection, DBusMessage * message, void *user_data)
///////////////////////////////////////////////////////////////////////////////
{
	UNUSED_PARAMETER(user_data);

	if (dbus_message_is_signal(message, DBUS_INTERFACE_LOCAL, "Disconnected")) {
		dbus_connection_unref(connection);
		connection = NULL;
	}
	return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
}

int hf_plugin_alp_init(void** pag)
{
	int result;
	DBusError error;

	*pag = NULL;

	dbus_error_init(&error);

	g_message("%s [%s %s] starting ...", __FUNCTION__, __DATE__, __TIME__);
	connection = dbus_bus_get(DBUS_BUS_SYSTEM, &error);
	if (connection == NULL)
	{
		g_critical("hshfgd: *** Failed to open connection to activating message bus: %s", error.message);
		goto failed;
	}

	g_message("hshfgd: Register object path");
	if (!dbus_connection_register_object_path(connection, hshfserver_object_path, &dbus_vtable_hshfserver, NULL))
	{
		g_critical("hshfgd: *** Failed to register object path");
		goto failed;
	}

	g_message("hshfgd: Acquire service: " HSHF_SERVICE_NAME);
	result = dbus_bus_request_name(connection, HSHF_SERVICE_NAME, 0, &error);
	if (dbus_error_is_set(&error))
	{
		g_critical("hshfgd: *** Failed to acquire service: %s", error.message);
		goto failed;
	}

	dbus_error_free(&error);

	return 0;

	failed:
		dbus_error_free(&error);

	return -ENOTCONN;
}

void hf_plugin_alp_exit(void* ag)
{
	g_message("%s [%s %s] exiting ...", __FUNCTION__, __DATE__, __TIME__);
}


void hf_plugin_alp_state_changed(int state, int type)
{
	gchar *stateP = NULL;
	gchar *profileP;

	g_message("%s", __FUNCTION__);

	switch (state)
	{
		case HEADSET_STATE_DISCONNECTED:
			stateP = HSHF_DISCONNECTED;
			break;
		case HEADSET_STATE_CONNECT_IN_PROGRESS:
			stateP = HSHF_CONNECTING;
			break;
		case HEADSET_STATE_CONNECTED:
			stateP = HSHF_CONNECTED;
			break;
		case HEADSET_STATE_PLAY_IN_PROGRESS:
			stateP = HSHF_AUDIO_CONNECTING;
			break;
		case HEADSET_STATE_PLAYING:
			stateP = HSHF_AUDIO_CONNECTED;
			break;
		default:
			stateP = HSHF_DISCONNECTED;
	}
	switch (type)
	{
		case SVC_HEADSET:
			profileP = HEADSET_PROFILE;
			break;
		case SVC_HANDSFREE:
			profileP = HANDSFREE_PROFILE;
			break;
		default:
			profileP = NO_PROFILE;
	}
	if(connection && hshfclient_name && hshfclient_object_path)
	{
		g_message("%s", __FUNCTION__);

		DBusMessage* msg = dbus_message_new_method_call(hshfclient_name, hshfclient_object_path, HSHFCLIENT_INTERFACE_NAME, "StateChange");
		if(msg) {
			if (dbus_message_append_args (msg,
						G_TYPE_STRING, stateP,
						G_TYPE_STRING, profileP,
						G_TYPE_UCHAR, 0,
						DBUS_TYPE_INVALID)) {
				// In case this gets fired off after we've disconnected.
				if (dbus_connection_get_is_connected (connection)) {
					dbus_connection_send (connection, msg, NULL);
				}
			}
			dbus_message_unref(msg);
		}

	}
}



